@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import string
 import re
 import responses
@@ -1736,7 +1734,7 @@ class APIGatewayBackend(BaseBackend):
         if not path_part:
             # We're attempting to create the default resource, which already exists.
             return api.default
-        if not re.match("^\\{?[a-zA-Z0-9._-]+\\+?\\}?$", path_part):
+        if not re.match("^\\{?[a-zA-Z0-9._\\-\\:]+\\+?\\}?$", path_part):
             raise InvalidResourcePathException()
         return api.add_child(path=path_part, parent_id=parent_resource_id)
 
