@@ -93,3 +93,33 @@ class UnknownProtocol(APIGatewayV2Error):
             "BadRequestException",
             "Invalid protocol specified. Must be one of [HTTP, WEBSOCKET]",
         )
+
+
+class DomainNameNotFound(APIGatewayV2Error):
+    code = 404
+
+    def __init__(self) -> None:
+        super().__init__(
+            "NotFoundException",
+            "The domain name resource specified in the request was not found.",
+        )
+
+
+class DomainNameAlreadyExists(APIGatewayV2Error):
+    code = 409
+
+    def __init__(self) -> None:
+        super().__init__(
+            "ConflictException",
+            "The domain name resource already exists.",
+        )
+
+
+class ApiMappingNotFound(APIGatewayV2Error):
+    code = 404
+
+    def __init__(self) -> None:
+        super().__init__(
+            "NotFoundException",
+            "The api mapping resource specified in the request was not found.",
+        )
