@@ -54,15 +54,12 @@ from .utils import (
 from moto.sqs import sqs_backends
 from moto.dynamodb import dynamodb_backends
 from moto.dynamodbstreams import dynamodbstreams_backends
-from moto.utilities.docker_utilities import DockerModel, parse_image_ref
+from moto.utilities.docker_utilities import DockerModel
 from tempfile import TemporaryDirectory
 
 logger = logging.getLogger(__name__)
 
 
-docker_3 = docker.__version__[0] >= "3"
-
-
 def zip2tar(zip_bytes: bytes) -> bytes:
     with TemporaryDirectory() as td:
         tarname = os.path.join(td, "data.tar")
@@ -128,21 +125,15 @@ class _DockerDataVolumeContext:
             self._vol_ref.volume = self._lambda_func.docker_client.volumes.create(
                 self._lambda_func.code_digest
             )
-            volumes = {
-                self.name: {"bind": "/tmp/data", "mode": "rw"}
-                if docker_3
-                else "/tmp/data"
-            }
+            volumes = {self.name: {"bind": settings.LAMBDA_DATA_DIR, "mode": "rw"}}
 
-            self._lambda_func.docker_client.images.pull(
-                ":".join(parse_image_ref("alpine"))
-            )
+            self._lambda_func.ensure_image_exists("busybox")
             container = self._lambda_func.docker_client.containers.run(
-                "alpine", "sleep 100", volumes=volumes, detach=True
+                "busybox", "sleep 100", volumes=volumes, detach=True
             )
             try:
                 tar_bytes = zip2tar(self._lambda_func.code_bytes)
-                container.put_archive("/tmp/data", tar_bytes)
+                container.put_archive(settings.LAMBDA_DATA_DIR, tar_bytes)
             finally:
                 container.remove(force=True)
 
@@ -782,9 +773,8 @@ class LambdaFunction(CloudFormationModel, DockerModel):
                     )
                     for image_repo in image_repos:
                         image_ref = f"{image_repo}:{self.run_time}"
-                        full_ref = ":".join(parse_image_ref(image_ref))
                         try:
-                            self.docker_client.images.pull(full_ref)
+                            self.ensure_image_exists(image_ref)
                             break
                         except docker.errors.NotFound:
                             pass
@@ -802,14 +792,11 @@ class LambdaFunction(CloudFormationModel, DockerModel):
                 finally:
                     if container:
                         try:
-                            exit_code = container.wait(timeout=300)
+                            exit_code = container.wait(timeout=300)["StatusCode"]
                         except requests.exceptions.ReadTimeout:
                             exit_code = -1
                             container.stop()
                             container.kill()
-                        else:
-                            if docker_3:
-                                exit_code = exit_code["StatusCode"]
 
                         output = container.logs(stdout=False, stderr=True)
                         output += container.logs(stdout=True, stderr=False)
@@ -830,6 +817,7 @@ class LambdaFunction(CloudFormationModel, DockerModel):
         except docker.errors.DockerException as e:
             # Docker itself is probably not running - there will be no Lambda-logs to handle
             msg = f"error running docker: {e}"
+            logger.error(msg)
             self.save_logs(msg)
             return msg, True, ""
 
@@ -1514,6 +1502,12 @@ class LambdaBackend(BaseBackend):
 
         MOTO_DOCKER_LAMBDA_IMAGE=mlupin/docker-lambda
 
+    Use the following environment variable if you want to configure the data directory used by the Docker containers:
+
+    .. sourcecode:: bash
+
+        MOTO_LAMBDA_DATA_DIR=/tmp/data
+
     .. note:: When using the decorators, a Docker container cannot reach Moto, as the Docker-container loses all mock-context. Any boto3-invocations used within your Lambda will try to connect to AWS.
     """
 
