@@ -95,6 +95,12 @@ class RouteTable(TaggedEC2Resource, CloudFormationModel):
             return self.all_associations_ids
         elif filter_name == "association.subnet-id":
             return self.associations.values()
+        elif filter_name == "route.destination-cidr-block":
+            return [
+                route.destination_cidr_block
+                for route in self.routes.values()
+                if route.destination_cidr_block is not None
+            ]
         elif filter_name == "route.gateway-id":
             return [
                 route.gateway.id
@@ -435,9 +441,6 @@ class RouteBackend:
         )
         route = route_table.routes[route_id]
 
-        if interface_id:
-            self.raise_not_implemented_error("ReplaceRoute to NetworkInterfaceId")  # type: ignore[attr-defined]
-
         route.gateway = None
         route.nat_gateway = None
         route.egress_only_igw = None
@@ -460,7 +463,7 @@ class RouteBackend:
             )
 
         route.instance = self.get_instance(instance_id) if instance_id else None  # type: ignore[attr-defined]
-        route.interface = None
+        route.interface = self.get_network_interface(interface_id) if interface_id else None  # type: ignore[attr-defined]
         route.vpc_pcx = (
             self.get_vpc_peering_connection(vpc_peering_connection_id)  # type: ignore[attr-defined]
             if vpc_peering_connection_id
@@ -508,7 +511,7 @@ class RouteBackend:
         for route in route_table.routes.values():
             if not route.destination_cidr_block:
                 continue
-            if not route.local and ip_v4_network.overlaps(
-                ipaddress.IPv4Network(str(route.destination_cidr_block))
+            if not route.local and ip_v4_network == ipaddress.IPv4Network(
+                str(route.destination_cidr_block)
             ):
                 raise RouteAlreadyExistsError(destination_cidr_block)
