@@ -1198,6 +1198,11 @@ class EC2ContainerServiceBackend(BaseBackend):
         launch_type: Optional[str],
         networking_configuration: Optional[Dict[str, Any]] = None,
     ) -> List[Task]:
+        if launch_type and launch_type not in ["EC2", "FARGATE", "EXTERNAL"]:
+            raise InvalidParameterException(
+                "launch type should be one of [EC2,FARGATE,EXTERNAL]"
+            )
+
         cluster = self._get_cluster(cluster_str)
 
         task_definition = self.describe_task_definition(task_definition_str)
@@ -2006,7 +2011,7 @@ class EC2ContainerServiceBackend(BaseBackend):
     def tag_resource(self, resource_arn: str, tags: List[Dict[str, str]]) -> None:
         parsed_arn = self._parse_resource_arn(resource_arn)
         resource = self._get_resource(resource_arn, parsed_arn)
-        resource.tags = self._merge_tags(resource.tags, tags)
+        resource.tags = self._merge_tags(resource.tags or [], tags)
 
     def _merge_tags(
         self, existing_tags: List[Dict[str, str]], new_tags: List[Dict[str, str]]
