@@ -1,16 +1,22 @@
 from collections import defaultdict
-from typing import Any
+from typing import Any, Dict, List, Iterable
 
 from moto.core.common_types import TYPE_RESPONSE
 from moto.core.responses import BaseResponse
 from moto.ec2.models import ec2_backends
 from moto.neptune.responses import NeptuneResponse
+from moto.neptune.responses import (
+    CREATE_GLOBAL_CLUSTER_TEMPLATE,
+    DESCRIBE_GLOBAL_CLUSTERS_TEMPLATE,
+    DELETE_GLOBAL_CLUSTER_TEMPLATE,
+    REMOVE_FROM_GLOBAL_CLUSTER_TEMPLATE,
+)
 from .models import rds_backends, RDSBackend
 from .exceptions import DBParameterGroupNotFoundError
 
 
 class RDSResponse(BaseResponse):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__(service_name="rds")
         # Neptune and RDS share a HTTP endpoint RDS is the lucky guy that catches all requests
         # So we have to determine whether we can handle an incoming request here, or whether it needs redirecting to Neptune
@@ -25,13 +31,13 @@ class RDSResponse(BaseResponse):
         self.neptune.setup_class(request, full_url, headers)
         return super()._dispatch(request, full_url, headers)
 
-    def __getattribute__(self, name: str):
-        if name in ["create_db_cluster"]:
+    def __getattribute__(self, name: str) -> Any:
+        if name in ["create_db_cluster", "create_global_cluster"]:
             if self._get_param("Engine") == "neptune":
                 return object.__getattribute__(self.neptune, name)
         return object.__getattribute__(self, name)
 
-    def _get_db_kwargs(self):
+    def _get_db_kwargs(self) -> Dict[str, Any]:
         args = {
             "auto_minor_version_upgrade": self._get_param("AutoMinorVersionUpgrade"),
             "allocated_storage": self._get_int_param("AllocatedStorage"),
@@ -84,7 +90,7 @@ class RDSResponse(BaseResponse):
         args["tags"] = self.unpack_list_params("Tags", "Tag")
         return args
 
-    def _get_modify_db_cluster_kwargs(self):
+    def _get_modify_db_cluster_kwargs(self) -> Dict[str, Any]:
         args = {
             "auto_minor_version_upgrade": self._get_param("AutoMinorVersionUpgrade"),
             "allocated_storage": self._get_int_param("AllocatedStorage"),
@@ -105,11 +111,12 @@ class RDSResponse(BaseResponse):
             "engine": self._get_param("Engine"),
             "engine_version": self._get_param("EngineVersion"),
             "enable_cloudwatch_logs_exports": self._get_params().get(
-                "EnableCloudwatchLogsExports"
+                "CloudwatchLogsExportConfiguration"
             ),
             "enable_iam_database_authentication": self._get_bool_param(
                 "EnableIAMDatabaseAuthentication"
             ),
+            "enable_http_endpoint": self._get_param("EnableHttpEndpoint"),
             "license_model": self._get_param("LicenseModel"),
             "iops": self._get_int_param("Iops"),
             "kms_key_id": self._get_param("KmsKeyId"),
@@ -139,7 +146,7 @@ class RDSResponse(BaseResponse):
         args["tags"] = self.unpack_list_params("Tags", "Tag")
         return args
 
-    def _get_db_replica_kwargs(self):
+    def _get_db_replica_kwargs(self) -> Dict[str, Any]:
         return {
             "auto_minor_version_upgrade": self._get_param("AutoMinorVersionUpgrade"),
             "availability_zone": self._get_param("AvailabilityZone"),
@@ -154,7 +161,7 @@ class RDSResponse(BaseResponse):
             "storage_type": self._get_param("StorageType"),
         }
 
-    def _get_option_group_kwargs(self):
+    def _get_option_group_kwargs(self) -> Dict[str, Any]:
         return {
             "major_engine_version": self._get_param("MajorEngineVersion"),
             "description": self._get_param("OptionGroupDescription"),
@@ -162,7 +169,7 @@ class RDSResponse(BaseResponse):
             "name": self._get_param("OptionGroupName"),
         }
 
-    def _get_db_parameter_group_kwargs(self):
+    def _get_db_parameter_group_kwargs(self) -> Dict[str, Any]:
         return {
             "description": self._get_param("Description"),
             "family": self._get_param("DBParameterGroupFamily"),
@@ -170,7 +177,7 @@ class RDSResponse(BaseResponse):
             "tags": self.unpack_list_params("Tags", "Tag"),
         }
 
-    def _get_db_cluster_kwargs(self):
+    def _get_db_cluster_kwargs(self) -> Dict[str, Any]:
         return {
             "availability_zones": self._get_multi_param(
                 "AvailabilityZones.AvailabilityZone"
@@ -180,25 +187,33 @@ class RDSResponse(BaseResponse):
             ),
             "db_name": self._get_param("DatabaseName"),
             "db_cluster_identifier": self._get_param("DBClusterIdentifier"),
+            "db_subnet_group_name": self._get_param("DBSubnetGroupName"),
             "deletion_protection": self._get_bool_param("DeletionProtection"),
             "engine": self._get_param("Engine"),
             "engine_version": self._get_param("EngineVersion"),
             "engine_mode": self._get_param("EngineMode"),
             "allocated_storage": self._get_param("AllocatedStorage"),
+            "global_cluster_identifier": self._get_param("GlobalClusterIdentifier"),
             "iops": self._get_param("Iops"),
             "storage_type": self._get_param("StorageType"),
+            "kms_key_id": self._get_param("KmsKeyId"),
             "master_username": self._get_param("MasterUsername"),
             "master_user_password": self._get_param("MasterUserPassword"),
+            "network_type": self._get_param("NetworkType"),
             "port": self._get_param("Port"),
-            "parameter_group": self._get_param("DBClusterParameterGroup"),
+            "parameter_group": self._get_param("DBClusterParameterGroupName"),
             "region": self.region,
             "db_cluster_instance_class": self._get_param("DBClusterInstanceClass"),
             "enable_http_endpoint": self._get_param("EnableHttpEndpoint"),
             "copy_tags_to_snapshot": self._get_param("CopyTagsToSnapshot"),
             "tags": self.unpack_list_params("Tags", "Tag"),
+            "scaling_configuration": self._get_dict_param("ScalingConfiguration."),
+            "replication_source_identifier": self._get_param(
+                "ReplicationSourceIdentifier"
+            ),
         }
 
-    def _get_export_task_kwargs(self):
+    def _get_export_task_kwargs(self) -> Dict[str, Any]:
         return {
             "export_task_identifier": self._get_param("ExportTaskIdentifier"),
             "source_arn": self._get_param("SourceArn"),
@@ -209,7 +224,7 @@ class RDSResponse(BaseResponse):
             "export_only": self.unpack_list_params("ExportOnly", "member"),
         }
 
-    def _get_event_subscription_kwargs(self):
+    def _get_event_subscription_kwargs(self) -> Dict[str, Any]:
         return {
             "subscription_name": self._get_param("SubscriptionName"),
             "sns_topic_arn": self._get_param("SnsTopicArn"),
@@ -222,29 +237,31 @@ class RDSResponse(BaseResponse):
             "tags": self.unpack_list_params("Tags", "Tag"),
         }
 
-    def unpack_list_params(self, label, child_label):
+    def unpack_list_params(self, label: str, child_label: str) -> List[Dict[str, Any]]:
         root = self._get_multi_param_dict(label) or {}
         return root.get(child_label, [])
 
-    def create_db_instance(self):
+    def create_db_instance(self) -> str:
         db_kwargs = self._get_db_kwargs()
         database = self.backend.create_db_instance(db_kwargs)
         template = self.response_template(CREATE_DATABASE_TEMPLATE)
         return template.render(database=database)
 
-    def create_db_instance_read_replica(self):
+    def create_db_instance_read_replica(self) -> str:
         db_kwargs = self._get_db_replica_kwargs()
 
         database = self.backend.create_db_instance_read_replica(db_kwargs)
         template = self.response_template(CREATE_DATABASE_REPLICA_TEMPLATE)
         return template.render(database=database)
 
-    def describe_db_instances(self):
+    def describe_db_instances(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         filters = self._get_multi_param("Filters.Filter.")
-        filters = {f["Name"]: f["Values"] for f in filters}
+        filter_dict = {f["Name"]: f["Values"] for f in filters}
         all_instances = list(
-            self.backend.describe_db_instances(db_instance_identifier, filters=filters)
+            self.backend.describe_db_instances(
+                db_instance_identifier, filters=filter_dict
+            )
         )
         marker = self._get_param("Marker")
         all_ids = [instance.db_instance_identifier for instance in all_instances]
@@ -263,7 +280,7 @@ class RDSResponse(BaseResponse):
         template = self.response_template(DESCRIBE_DATABASES_TEMPLATE)
         return template.render(databases=instances_resp, marker=next_marker)
 
-    def modify_db_instance(self):
+    def modify_db_instance(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         db_kwargs = self._get_db_kwargs()
         # NOTE modify_db_instance does not support tags
@@ -275,7 +292,7 @@ class RDSResponse(BaseResponse):
         template = self.response_template(MODIFY_DATABASE_TEMPLATE)
         return template.render(database=database)
 
-    def delete_db_instance(self):
+    def delete_db_instance(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         db_snapshot_name = self._get_param("FinalDBSnapshotIdentifier")
         database = self.backend.delete_db_instance(
@@ -284,13 +301,13 @@ class RDSResponse(BaseResponse):
         template = self.response_template(DELETE_DATABASE_TEMPLATE)
         return template.render(database=database)
 
-    def reboot_db_instance(self):
+    def reboot_db_instance(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         database = self.backend.reboot_db_instance(db_instance_identifier)
         template = self.response_template(REBOOT_DATABASE_TEMPLATE)
         return template.render(database=database)
 
-    def create_db_snapshot(self):
+    def create_db_snapshot(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         db_snapshot_identifier = self._get_param("DBSnapshotIdentifier")
         tags = self.unpack_list_params("Tags", "Tag")
@@ -300,7 +317,7 @@ class RDSResponse(BaseResponse):
         template = self.response_template(CREATE_SNAPSHOT_TEMPLATE)
         return template.render(snapshot=snapshot)
 
-    def copy_db_snapshot(self):
+    def copy_db_snapshot(self) -> str:
         source_snapshot_identifier = self._get_param("SourceDBSnapshotIdentifier")
         target_snapshot_identifier = self._get_param("TargetDBSnapshotIdentifier")
         tags = self.unpack_list_params("Tags", "Tag")
@@ -310,18 +327,18 @@ class RDSResponse(BaseResponse):
         template = self.response_template(COPY_SNAPSHOT_TEMPLATE)
         return template.render(snapshot=snapshot)
 
-    def describe_db_snapshots(self):
+    def describe_db_snapshots(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         db_snapshot_identifier = self._get_param("DBSnapshotIdentifier")
         filters = self._get_multi_param("Filters.Filter.")
-        filters = {f["Name"]: f["Values"] for f in filters}
-        snapshots = self.backend.describe_database_snapshots(
-            db_instance_identifier, db_snapshot_identifier, filters
+        filter_dict = {f["Name"]: f["Values"] for f in filters}
+        snapshots = self.backend.describe_db_snapshots(
+            db_instance_identifier, db_snapshot_identifier, filter_dict
         )
         template = self.response_template(DESCRIBE_SNAPSHOTS_TEMPLATE)
         return template.render(snapshots=snapshots)
 
-    def promote_read_replica(self):
+    def promote_read_replica(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         db_kwargs = self._get_db_kwargs()
         database = self.backend.promote_read_replica(db_kwargs)
@@ -329,13 +346,13 @@ class RDSResponse(BaseResponse):
         template = self.response_template(PROMOTE_REPLICA_TEMPLATE)
         return template.render(database=database)
 
-    def delete_db_snapshot(self):
+    def delete_db_snapshot(self) -> str:
         db_snapshot_identifier = self._get_param("DBSnapshotIdentifier")
         snapshot = self.backend.delete_db_snapshot(db_snapshot_identifier)
         template = self.response_template(DELETE_SNAPSHOT_TEMPLATE)
         return template.render(snapshot=snapshot)
 
-    def restore_db_instance_from_db_snapshot(self):
+    def restore_db_instance_from_db_snapshot(self) -> str:
         db_snapshot_identifier = self._get_param("DBSnapshotIdentifier")
         db_kwargs = self._get_db_kwargs()
         new_instance = self.backend.restore_db_instance_from_db_snapshot(
@@ -344,27 +361,27 @@ class RDSResponse(BaseResponse):
         template = self.response_template(RESTORE_INSTANCE_FROM_SNAPSHOT_TEMPLATE)
         return template.render(database=new_instance)
 
-    def list_tags_for_resource(self):
+    def list_tags_for_resource(self) -> str:
         arn = self._get_param("ResourceName")
         template = self.response_template(LIST_TAGS_FOR_RESOURCE_TEMPLATE)
         tags = self.backend.list_tags_for_resource(arn)
         return template.render(tags=tags)
 
-    def add_tags_to_resource(self):
+    def add_tags_to_resource(self) -> str:
         arn = self._get_param("ResourceName")
         tags = self.unpack_list_params("Tags", "Tag")
         tags = self.backend.add_tags_to_resource(arn, tags)
         template = self.response_template(ADD_TAGS_TO_RESOURCE_TEMPLATE)
         return template.render(tags=tags)
 
-    def remove_tags_from_resource(self):
+    def remove_tags_from_resource(self) -> str:
         arn = self._get_param("ResourceName")
         tag_keys = self.unpack_list_params("TagKeys", "member")
-        self.backend.remove_tags_from_resource(arn, tag_keys)
+        self.backend.remove_tags_from_resource(arn, tag_keys)  # type: ignore
         template = self.response_template(REMOVE_TAGS_FROM_RESOURCE_TEMPLATE)
         return template.render()
 
-    def stop_db_instance(self):
+    def stop_db_instance(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         db_snapshot_identifier = self._get_param("DBSnapshotIdentifier")
         database = self.backend.stop_db_instance(
@@ -373,13 +390,13 @@ class RDSResponse(BaseResponse):
         template = self.response_template(STOP_DATABASE_TEMPLATE)
         return template.render(database=database)
 
-    def start_db_instance(self):
+    def start_db_instance(self) -> str:
         db_instance_identifier = self._get_param("DBInstanceIdentifier")
         database = self.backend.start_db_instance(db_instance_identifier)
         template = self.response_template(START_DATABASE_TEMPLATE)
         return template.render(database=database)
 
-    def create_db_security_group(self):
+    def create_db_security_group(self) -> str:
         group_name = self._get_param("DBSecurityGroupName")
         description = self._get_param("DBSecurityGroupDescription")
         tags = self.unpack_list_params("Tags", "Tag")
@@ -389,19 +406,19 @@ class RDSResponse(BaseResponse):
         template = self.response_template(CREATE_SECURITY_GROUP_TEMPLATE)
         return template.render(security_group=security_group)
 
-    def describe_db_security_groups(self):
+    def describe_db_security_groups(self) -> str:
         security_group_name = self._get_param("DBSecurityGroupName")
         security_groups = self.backend.describe_security_groups(security_group_name)
         template = self.response_template(DESCRIBE_SECURITY_GROUPS_TEMPLATE)
         return template.render(security_groups=security_groups)
 
-    def delete_db_security_group(self):
+    def delete_db_security_group(self) -> str:
         security_group_name = self._get_param("DBSecurityGroupName")
         security_group = self.backend.delete_security_group(security_group_name)
         template = self.response_template(DELETE_SECURITY_GROUP_TEMPLATE)
         return template.render(security_group=security_group)
 
-    def authorize_db_security_group_ingress(self):
+    def authorize_db_security_group_ingress(self) -> str:
         security_group_name = self._get_param("DBSecurityGroupName")
         cidr_ip = self._get_param("CIDRIP")
         security_group = self.backend.authorize_security_group(
@@ -410,7 +427,7 @@ class RDSResponse(BaseResponse):
         template = self.response_template(AUTHORIZE_SECURITY_GROUP_TEMPLATE)
         return template.render(security_group=security_group)
 
-    def create_db_subnet_group(self):
+    def create_db_subnet_group(self) -> str:
         subnet_name = self._get_param("DBSubnetGroupName")
         description = self._get_param("DBSubnetGroupDescription")
         subnet_ids = self._get_multi_param("SubnetIds.SubnetIdentifier")
@@ -425,13 +442,13 @@ class RDSResponse(BaseResponse):
         template = self.response_template(CREATE_SUBNET_GROUP_TEMPLATE)
         return template.render(subnet_group=subnet_group)
 
-    def describe_db_subnet_groups(self):
+    def describe_db_subnet_groups(self) -> str:
         subnet_name = self._get_param("DBSubnetGroupName")
-        subnet_groups = self.backend.describe_subnet_groups(subnet_name)
+        subnet_groups = self.backend.describe_db_subnet_groups(subnet_name)
         template = self.response_template(DESCRIBE_SUBNET_GROUPS_TEMPLATE)
         return template.render(subnet_groups=subnet_groups)
 
-    def modify_db_subnet_group(self):
+    def modify_db_subnet_group(self) -> str:
         subnet_name = self._get_param("DBSubnetGroupName")
         description = self._get_param("DBSubnetGroupDescription")
         subnet_ids = self._get_multi_param("SubnetIds.SubnetIdentifier")
@@ -445,25 +462,25 @@ class RDSResponse(BaseResponse):
         template = self.response_template(MODIFY_SUBNET_GROUPS_TEMPLATE)
         return template.render(subnet_group=subnet_group)
 
-    def delete_db_subnet_group(self):
+    def delete_db_subnet_group(self) -> str:
         subnet_name = self._get_param("DBSubnetGroupName")
         subnet_group = self.backend.delete_subnet_group(subnet_name)
         template = self.response_template(DELETE_SUBNET_GROUP_TEMPLATE)
         return template.render(subnet_group=subnet_group)
 
-    def create_option_group(self):
+    def create_option_group(self) -> str:
         kwargs = self._get_option_group_kwargs()
         option_group = self.backend.create_option_group(kwargs)
         template = self.response_template(CREATE_OPTION_GROUP_TEMPLATE)
         return template.render(option_group=option_group)
 
-    def delete_option_group(self):
+    def delete_option_group(self) -> str:
         kwargs = self._get_option_group_kwargs()
         option_group = self.backend.delete_option_group(kwargs["name"])
         template = self.response_template(DELETE_OPTION_GROUP_TEMPLATE)
         return template.render(option_group=option_group)
 
-    def describe_option_groups(self):
+    def describe_option_groups(self) -> str:
         kwargs = self._get_option_group_kwargs()
         kwargs["max_records"] = self._get_int_param("MaxRecords")
         kwargs["marker"] = self._get_param("Marker")
@@ -471,15 +488,14 @@ class RDSResponse(BaseResponse):
         template = self.response_template(DESCRIBE_OPTION_GROUP_TEMPLATE)
         return template.render(option_groups=option_groups)
 
-    def describe_option_group_options(self):
+    def describe_option_group_options(self) -> str:
         engine_name = self._get_param("EngineName")
         major_engine_version = self._get_param("MajorEngineVersion")
-        option_group_options = self.backend.describe_option_group_options(
+        return self.backend.describe_option_group_options(
             engine_name, major_engine_version
         )
-        return option_group_options
 
-    def modify_option_group(self):
+    def modify_option_group(self) -> str:
         option_group_name = self._get_param("OptionGroupName")
         count = 1
         options_to_include = []
@@ -515,13 +531,13 @@ class RDSResponse(BaseResponse):
         template = self.response_template(MODIFY_OPTION_GROUP_TEMPLATE)
         return template.render(option_group=option_group)
 
-    def create_db_parameter_group(self):
+    def create_db_parameter_group(self) -> str:
         kwargs = self._get_db_parameter_group_kwargs()
         db_parameter_group = self.backend.create_db_parameter_group(kwargs)
         template = self.response_template(CREATE_DB_PARAMETER_GROUP_TEMPLATE)
         return template.render(db_parameter_group=db_parameter_group)
 
-    def describe_db_parameter_groups(self):
+    def describe_db_parameter_groups(self) -> str:
         kwargs = self._get_db_parameter_group_kwargs()
         kwargs["max_records"] = self._get_int_param("MaxRecords")
         kwargs["marker"] = self._get_param("Marker")
@@ -529,7 +545,7 @@ class RDSResponse(BaseResponse):
         template = self.response_template(DESCRIBE_DB_PARAMETER_GROUPS_TEMPLATE)
         return template.render(db_parameter_groups=db_parameter_groups)
 
-    def modify_db_parameter_group(self):
+    def modify_db_parameter_group(self) -> str:
         db_parameter_group_name = self._get_param("DBParameterGroupName")
         db_parameter_group_parameters = self._get_db_parameter_group_parameters()
         db_parameter_group = self.backend.modify_db_parameter_group(
@@ -538,8 +554,8 @@ class RDSResponse(BaseResponse):
         template = self.response_template(MODIFY_DB_PARAMETER_GROUP_TEMPLATE)
         return template.render(db_parameter_group=db_parameter_group)
 
-    def _get_db_parameter_group_parameters(self):
-        parameter_group_parameters = defaultdict(dict)
+    def _get_db_parameter_group_parameters(self) -> Iterable[Dict[str, Any]]:
+        parameter_group_parameters: Dict[str, Any] = defaultdict(dict)
         for param_name, value in self.querystring.items():
             if not param_name.startswith("Parameters.Parameter"):
                 continue
@@ -552,7 +568,7 @@ class RDSResponse(BaseResponse):
 
         return parameter_group_parameters.values()
 
-    def describe_db_parameters(self):
+    def describe_db_parameters(self) -> str:
         db_parameter_group_name = self._get_param("DBParameterGroupName")
         db_parameter_groups = self.backend.describe_db_parameter_groups(
             {"name": db_parameter_group_name}
@@ -563,31 +579,44 @@ class RDSResponse(BaseResponse):
         template = self.response_template(DESCRIBE_DB_PARAMETERS_TEMPLATE)
         return template.render(db_parameter_group=db_parameter_groups[0])
 
-    def delete_db_parameter_group(self):
+    def delete_db_parameter_group(self) -> str:
         kwargs = self._get_db_parameter_group_kwargs()
         db_parameter_group = self.backend.delete_db_parameter_group(kwargs["name"])
         template = self.response_template(DELETE_DB_PARAMETER_GROUP_TEMPLATE)
         return template.render(db_parameter_group=db_parameter_group)
 
-    def create_db_cluster(self):
+    def describe_db_cluster_parameters(self) -> str:
+        db_parameter_group_name = self._get_param("DBParameterGroupName")
+        db_parameter_groups = self.backend.describe_db_cluster_parameters()
+        if db_parameter_groups is None:
+            raise DBParameterGroupNotFoundError(db_parameter_group_name)
+
+        template = self.response_template(DESCRIBE_DB_CLUSTER_PARAMETERS_TEMPLATE)
+        return template.render(db_parameter_group=db_parameter_groups)
+
+    def create_db_cluster(self) -> str:
         kwargs = self._get_db_cluster_kwargs()
         cluster = self.backend.create_db_cluster(kwargs)
         template = self.response_template(CREATE_DB_CLUSTER_TEMPLATE)
         return template.render(cluster=cluster)
 
-    def modify_db_cluster(self):
+    def modify_db_cluster(self) -> str:
         kwargs = self._get_modify_db_cluster_kwargs()
         cluster = self.backend.modify_db_cluster(kwargs)
         template = self.response_template(MODIFY_DB_CLUSTER_TEMPLATE)
         return template.render(cluster=cluster)
 
-    def describe_db_clusters(self):
+    def describe_db_clusters(self) -> str:
         _id = self._get_param("DBClusterIdentifier")
-        clusters = self.backend.describe_db_clusters(cluster_identifier=_id)
+        filters = self._get_multi_param("Filters.Filter.")
+        filter_dict = {f["Name"]: f["Values"] for f in filters}
+        clusters = self.backend.describe_db_clusters(
+            cluster_identifier=_id, filters=filter_dict
+        )
         template = self.response_template(DESCRIBE_CLUSTERS_TEMPLATE)
         return template.render(clusters=clusters)
 
-    def delete_db_cluster(self):
+    def delete_db_cluster(self) -> str:
         _id = self._get_param("DBClusterIdentifier")
         snapshot_name = self._get_param("FinalDBSnapshotIdentifier")
         cluster = self.backend.delete_db_cluster(
@@ -596,19 +625,19 @@ class RDSResponse(BaseResponse):
         template = self.response_template(DELETE_CLUSTER_TEMPLATE)
         return template.render(cluster=cluster)
 
-    def start_db_cluster(self):
+    def start_db_cluster(self) -> str:
         _id = self._get_param("DBClusterIdentifier")
         cluster = self.backend.start_db_cluster(cluster_identifier=_id)
         template = self.response_template(START_CLUSTER_TEMPLATE)
         return template.render(cluster=cluster)
 
-    def stop_db_cluster(self):
+    def stop_db_cluster(self) -> str:
         _id = self._get_param("DBClusterIdentifier")
         cluster = self.backend.stop_db_cluster(cluster_identifier=_id)
         template = self.response_template(STOP_CLUSTER_TEMPLATE)
         return template.render(cluster=cluster)
 
-    def create_db_cluster_snapshot(self):
+    def create_db_cluster_snapshot(self) -> str:
         db_cluster_identifier = self._get_param("DBClusterIdentifier")
         db_snapshot_identifier = self._get_param("DBClusterSnapshotIdentifier")
         tags = self.unpack_list_params("Tags", "Tag")
@@ -618,7 +647,7 @@ class RDSResponse(BaseResponse):
         template = self.response_template(CREATE_CLUSTER_SNAPSHOT_TEMPLATE)
         return template.render(snapshot=snapshot)
 
-    def copy_db_cluster_snapshot(self):
+    def copy_db_cluster_snapshot(self) -> str:
         source_snapshot_identifier = self._get_param(
             "SourceDBClusterSnapshotIdentifier"
         )
@@ -632,24 +661,24 @@ class RDSResponse(BaseResponse):
         template = self.response_template(COPY_CLUSTER_SNAPSHOT_TEMPLATE)
         return template.render(snapshot=snapshot)
 
-    def describe_db_cluster_snapshots(self):
+    def describe_db_cluster_snapshots(self) -> str:
         db_cluster_identifier = self._get_param("DBClusterIdentifier")
         db_snapshot_identifier = self._get_param("DBClusterSnapshotIdentifier")
         filters = self._get_multi_param("Filters.Filter.")
-        filters = {f["Name"]: f["Values"] for f in filters}
+        filter_dict = {f["Name"]: f["Values"] for f in filters}
         snapshots = self.backend.describe_db_cluster_snapshots(
-            db_cluster_identifier, db_snapshot_identifier, filters
+            db_cluster_identifier, db_snapshot_identifier, filter_dict
         )
         template = self.response_template(DESCRIBE_CLUSTER_SNAPSHOTS_TEMPLATE)
         return template.render(snapshots=snapshots)
 
-    def delete_db_cluster_snapshot(self):
+    def delete_db_cluster_snapshot(self) -> str:
         db_snapshot_identifier = self._get_param("DBClusterSnapshotIdentifier")
         snapshot = self.backend.delete_db_cluster_snapshot(db_snapshot_identifier)
         template = self.response_template(DELETE_CLUSTER_SNAPSHOT_TEMPLATE)
         return template.render(snapshot=snapshot)
 
-    def restore_db_cluster_from_snapshot(self):
+    def restore_db_cluster_from_snapshot(self) -> str:
         db_snapshot_identifier = self._get_param("SnapshotIdentifier")
         db_kwargs = self._get_db_cluster_kwargs()
         new_cluster = self.backend.restore_db_cluster_from_snapshot(
@@ -658,43 +687,43 @@ class RDSResponse(BaseResponse):
         template = self.response_template(RESTORE_CLUSTER_FROM_SNAPSHOT_TEMPLATE)
         return template.render(cluster=new_cluster)
 
-    def start_export_task(self):
+    def start_export_task(self) -> str:
         kwargs = self._get_export_task_kwargs()
         export_task = self.backend.start_export_task(kwargs)
         template = self.response_template(START_EXPORT_TASK_TEMPLATE)
         return template.render(task=export_task)
 
-    def cancel_export_task(self):
+    def cancel_export_task(self) -> str:
         export_task_identifier = self._get_param("ExportTaskIdentifier")
         export_task = self.backend.cancel_export_task(export_task_identifier)
         template = self.response_template(CANCEL_EXPORT_TASK_TEMPLATE)
         return template.render(task=export_task)
 
-    def describe_export_tasks(self):
+    def describe_export_tasks(self) -> str:
         export_task_identifier = self._get_param("ExportTaskIdentifier")
         tasks = self.backend.describe_export_tasks(export_task_identifier)
         template = self.response_template(DESCRIBE_EXPORT_TASKS_TEMPLATE)
         return template.render(tasks=tasks)
 
-    def create_event_subscription(self):
+    def create_event_subscription(self) -> str:
         kwargs = self._get_event_subscription_kwargs()
         subscription = self.backend.create_event_subscription(kwargs)
         template = self.response_template(CREATE_EVENT_SUBSCRIPTION_TEMPLATE)
         return template.render(subscription=subscription)
 
-    def delete_event_subscription(self):
+    def delete_event_subscription(self) -> str:
         subscription_name = self._get_param("SubscriptionName")
         subscription = self.backend.delete_event_subscription(subscription_name)
         template = self.response_template(DELETE_EVENT_SUBSCRIPTION_TEMPLATE)
         return template.render(subscription=subscription)
 
-    def describe_event_subscriptions(self):
+    def describe_event_subscriptions(self) -> str:
         subscription_name = self._get_param("SubscriptionName")
         subscriptions = self.backend.describe_event_subscriptions(subscription_name)
         template = self.response_template(DESCRIBE_EVENT_SUBSCRIPTIONS_TEMPLATE)
         return template.render(subscriptions=subscriptions)
 
-    def describe_orderable_db_instance_options(self):
+    def describe_orderable_db_instance_options(self) -> str:
         engine = self._get_param("Engine")
         engine_version = self._get_param("EngineVersion")
         options = self.backend.describe_orderable_db_instance_options(
@@ -703,14 +732,74 @@ class RDSResponse(BaseResponse):
         template = self.response_template(DESCRIBE_ORDERABLE_CLUSTER_OPTIONS)
         return template.render(options=options, marker=None)
 
-    def describe_global_clusters(self):
-        return self.neptune.describe_global_clusters()
+    def describe_global_clusters(self) -> str:
+        clusters = self.backend.describe_global_clusters()
+        template = self.response_template(DESCRIBE_GLOBAL_CLUSTERS_TEMPLATE)
+        return template.render(clusters=clusters)
 
-    def create_global_cluster(self):
-        return self.neptune.create_global_cluster()
+    def create_global_cluster(self) -> str:
+        params = self._get_params()
+        cluster = self.backend.create_global_cluster(
+            global_cluster_identifier=params["GlobalClusterIdentifier"],
+            source_db_cluster_identifier=params.get("SourceDBClusterIdentifier"),
+            engine=params.get("Engine"),
+            engine_version=params.get("EngineVersion"),
+            storage_encrypted=params.get("StorageEncrypted"),
+            deletion_protection=params.get("DeletionProtection"),
+        )
+        template = self.response_template(CREATE_GLOBAL_CLUSTER_TEMPLATE)
+        return template.render(cluster=cluster)
 
-    def delete_global_cluster(self):
-        return self.neptune.delete_global_cluster()
+    def delete_global_cluster(self) -> str:
+        params = self._get_params()
+        cluster = self.backend.delete_global_cluster(
+            global_cluster_identifier=params["GlobalClusterIdentifier"],
+        )
+        template = self.response_template(DELETE_GLOBAL_CLUSTER_TEMPLATE)
+        return template.render(cluster=cluster)
+
+    def remove_from_global_cluster(self) -> str:
+        params = self._get_params()
+        global_cluster = self.backend.remove_from_global_cluster(
+            global_cluster_identifier=params["GlobalClusterIdentifier"],
+            db_cluster_identifier=params["DbClusterIdentifier"],
+        )
+        template = self.response_template(REMOVE_FROM_GLOBAL_CLUSTER_TEMPLATE)
+        return template.render(cluster=global_cluster)
+
+    def create_db_cluster_parameter_group(self) -> str:
+        group_name = self._get_param("DBClusterParameterGroupName")
+        family = self._get_param("DBParameterGroupFamily")
+        desc = self._get_param("Description")
+        db_cluster_parameter_group = self.backend.create_db_cluster_parameter_group(
+            group_name=group_name,
+            family=family,
+            description=desc,
+        )
+        template = self.response_template(CREATE_DB_CLUSTER_PARAMETER_GROUP_TEMPLATE)
+        return template.render(db_cluster_parameter_group=db_cluster_parameter_group)
+
+    def describe_db_cluster_parameter_groups(self) -> str:
+        group_name = self._get_param("DBClusterParameterGroupName")
+        db_parameter_groups = self.backend.describe_db_cluster_parameter_groups(
+            group_name=group_name,
+        )
+        template = self.response_template(DESCRIBE_DB_CLUSTER_PARAMETER_GROUPS_TEMPLATE)
+        return template.render(db_parameter_groups=db_parameter_groups)
+
+    def delete_db_cluster_parameter_group(self) -> str:
+        group_name = self._get_param("DBClusterParameterGroupName")
+        self.backend.delete_db_cluster_parameter_group(
+            group_name=group_name,
+        )
+        template = self.response_template(DELETE_DB_CLUSTER_PARAMETER_GROUP_TEMPLATE)
+        return template.render()
+
+    def promote_read_replica_db_cluster(self) -> str:
+        db_cluster_identifier = self._get_param("DBClusterIdentifier")
+        cluster = self.backend.promote_read_replica_db_cluster(db_cluster_identifier)
+        template = self.response_template(PROMOTE_READ_REPLICA_DB_CLUSTER_TEMPLATE)
+        return template.render(cluster=cluster)
 
 
 CREATE_DATABASE_TEMPLATE = """<CreateDBInstanceResponse xmlns="http://rds.amazonaws.com/doc/2014-09-01/">
@@ -1044,6 +1133,24 @@ DESCRIBE_DB_PARAMETERS_TEMPLATE = """<DescribeDBParametersResponse xmlns="http:/
 </DescribeDBParametersResponse>
 """
 
+DESCRIBE_DB_CLUSTER_PARAMETERS_TEMPLATE = """<DescribeDBClusterParametersResponse xmlns="http://rds.amazonaws.com/doc/2014-09-01/">
+  <DescribeDBClusterParametersResult>
+    <Parameters>
+      {%- for param in db_parameter_group -%}
+      <Parameter>
+        {%- for parameter_name, parameter_value in db_parameter.items() -%}
+        <{{ parameter_name }}>{{ parameter_value }}</{{ parameter_name }}>
+        {%- endfor -%}
+      </Parameter>
+      {%- endfor -%}
+    </Parameters>
+  </DescribeDBClusterParametersResult>
+  <ResponseMetadata>
+    <RequestId>8c40488f-b9ff-11d3-a15e-7ac49293f4fa</RequestId>
+  </ResponseMetadata>
+</DescribeDBClusterParametersResponse>
+"""
+
 LIST_TAGS_FOR_RESOURCE_TEMPLATE = """<ListTagsForResourceResponse xmlns="http://rds.amazonaws.com/doc/2014-10-31/">
   <ListTagsForResourceResult>
     <TagList>
@@ -1257,7 +1364,7 @@ DESCRIBE_ORDERABLE_CLUSTER_OPTIONS = """<DescribeOrderableDBInstanceOptionsRespo
     <OrderableDBInstanceOptions>
     {% for option in options %}
       <OrderableDBInstanceOption>
-        <OutpostCapable>option["OutpostCapable"]</OutpostCapable>
+        <OutpostCapable>false</OutpostCapable>
         <AvailabilityZones>
           {% for zone in option["AvailabilityZones"] %}
           <AvailabilityZone>
@@ -1302,3 +1409,41 @@ DESCRIBE_ORDERABLE_CLUSTER_OPTIONS = """<DescribeOrderableDBInstanceOptionsRespo
     <RequestId>54212dc5-16c4-4eb8-a88e-448691e877ab</RequestId>
   </ResponseMetadata>
 </DescribeOrderableDBInstanceOptionsResponse>"""
+
+CREATE_DB_CLUSTER_PARAMETER_GROUP_TEMPLATE = """<CreateDBClusterParameterGroupResponse xmlns="http://rds.amazonaws.com/doc/2014-09-01/">
+  <CreateDBClusterParameterGroupResult>
+    {{ db_cluster_parameter_group.to_xml() }}
+  </CreateDBClusterParameterGroupResult>
+  <ResponseMetadata>
+    <RequestId>7805c127-af22-11c3-96ac-6999cc5f7e72</RequestId>
+  </ResponseMetadata>
+</CreateDBClusterParameterGroupResponse>"""
+
+
+DESCRIBE_DB_CLUSTER_PARAMETER_GROUPS_TEMPLATE = """<DescribeDBClusterParameterGroupsResponse xmlns="http://rds.amazonaws.com/doc/2014-09-01/">
+  <DescribeDBClusterParameterGroupsResult>
+    <DBClusterParameterGroups>
+    {%- for db_parameter_group in db_parameter_groups -%}
+      {{ db_parameter_group.to_xml() }}
+    {%- endfor -%}
+    </DBClusterParameterGroups>
+  </DescribeDBClusterParameterGroupsResult>
+  <ResponseMetadata>
+    <RequestId>b75d527a-b98c-11d3-f272-7cd6cce12cc5</RequestId>
+  </ResponseMetadata>
+</DescribeDBClusterParameterGroupsResponse>"""
+
+DELETE_DB_CLUSTER_PARAMETER_GROUP_TEMPLATE = """<DeleteDBClusterParameterGroupResponse xmlns="http://rds.amazonaws.com/doc/2014-09-01/">
+  <ResponseMetadata>
+    <RequestId>cad6c267-ba25-11d3-fe11-33d33a9bb7e3</RequestId>
+  </ResponseMetadata>
+</DeleteDBClusterParameterGroupResponse>"""
+
+PROMOTE_READ_REPLICA_DB_CLUSTER_TEMPLATE = """<PromoteReadReplicaDBClusterResponse xmlns="http://rds.amazonaws.com/doc/2014-09-01/">
+  <PromoteReadReplicaDBClusterResult>
+    {{ cluster.to_xml() }}
+  </PromoteReadReplicaDBClusterResult>
+  <ResponseMetadata>
+    <RequestId>7369556f-b70d-11c3-faca-6ba18376ea1b</RequestId>
+  </ResponseMetadata>
+</PromoteReadReplicaDBClusterResponse>"""
