@@ -1,38 +1,40 @@
 import re
+from typing import Any, Dict, Optional, Tuple, Union
 
+from moto.core.common_types import TYPE_RESPONSE
 from moto.core.exceptions import RESTError
 from moto.core.responses import BaseResponse
 from moto.core.utils import underscores_to_camelcase, camelcase_to_pascal
 from moto.utilities.aws_headers import amz_crc32, amzn_request_id
 from urllib.parse import urlparse
 
+from .constants import (
+    DEFAULT_RECEIVED_MESSAGES,
+    MAXIMUM_MESSAGE_LENGTH,
+    MAXIMUM_VISIBILITY_TIMEOUT,
+)
 from .exceptions import (
     EmptyBatchRequest,
     InvalidAttributeName,
-    ReceiptHandleIsInvalid,
     BatchEntryIdsNotDistinct,
 )
-from .models import sqs_backends
+from .models import sqs_backends, SQSBackend
 from .utils import parse_message_attributes, extract_input_message_attributes
 
-MAXIMUM_VISIBILTY_TIMEOUT = 43200
-MAXIMUM_MESSAGE_LENGTH = 262144  # 256 KiB
-DEFAULT_RECEIVED_MESSAGES = 1
-
 
 class SQSResponse(BaseResponse):
 
     region_regex = re.compile(r"://(.+?)\.queue\.amazonaws\.com")
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__(service_name="sqs")
 
     @property
-    def sqs_backend(self):
+    def sqs_backend(self) -> SQSBackend:
         return sqs_backends[self.current_account][self.region]
 
     @property
-    def attribute(self):
+    def attribute(self) -> Any:  # type: ignore[misc]
         if not hasattr(self, "_attribute"):
             self._attribute = self._get_map_prefix(
                 "Attribute", key_end=".Name", value_end=".Value"
@@ -40,14 +42,14 @@ class SQSResponse(BaseResponse):
         return self._attribute
 
     @property
-    def tags(self):
+    def tags(self) -> Dict[str, str]:
         if not hasattr(self, "_tags"):
             self._tags = self._get_map_prefix("Tag", key_end=".Key", value_end=".Value")
         return self._tags
 
-    def _get_queue_name(self):
+    def _get_queue_name(self) -> str:
         try:
-            queue_url = self.querystring.get("QueueUrl")[0]
+            queue_url = self.querystring.get("QueueUrl")[0]  # type: ignore
             if queue_url.startswith("http://") or queue_url.startswith("https://"):
                 return queue_url.split("/")[-1]
             else:
@@ -57,24 +59,24 @@ class SQSResponse(BaseResponse):
             # Fallback to reading from the URL for botocore
             return self.path.split("/")[-1]
 
-    def _get_validated_visibility_timeout(self, timeout=None):
+    def _get_validated_visibility_timeout(self, timeout: Optional[str] = None) -> int:
         """
-        :raises ValueError: If specified visibility timeout exceeds MAXIMUM_VISIBILTY_TIMEOUT
+        :raises ValueError: If specified visibility timeout exceeds MAXIMUM_VISIBILITY_TIMEOUT
         :raises TypeError: If visibility timeout was not specified
         """
         if timeout is not None:
             visibility_timeout = int(timeout)
         else:
-            visibility_timeout = int(self.querystring.get("VisibilityTimeout")[0])
+            visibility_timeout = int(self.querystring.get("VisibilityTimeout")[0])  # type: ignore
 
-        if visibility_timeout > MAXIMUM_VISIBILTY_TIMEOUT:
+        if visibility_timeout > MAXIMUM_VISIBILITY_TIMEOUT:
             raise ValueError
 
         return visibility_timeout
 
     @amz_crc32  # crc last as request_id can edit XML
     @amzn_request_id
-    def call_action(self):
+    def call_action(self) -> TYPE_RESPONSE:
         status_code, headers, body = super().call_action()
         if status_code == 404:
             queue_name = self.querystring.get("QueueName", [""])[0]
@@ -83,11 +85,13 @@ class SQSResponse(BaseResponse):
             return 404, headers, response
         return status_code, headers, body
 
-    def _error(self, code, message, status=400):
+    def _error(
+        self, code: str, message: str, status: int = 400
+    ) -> Tuple[str, Dict[str, int]]:
         template = self.response_template(ERROR_TEMPLATE)
         return template.render(code=code, message=message), dict(status=status)
 
-    def create_queue(self):
+    def create_queue(self) -> str:
         request_url = urlparse(self.uri)
         queue_name = self._get_param("QueueName")
 
@@ -96,7 +100,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(CREATE_QUEUE_RESPONSE)
         return template.render(queue_url=queue.url(request_url))
 
-    def get_queue_url(self):
+    def get_queue_url(self) -> str:
         request_url = urlparse(self.uri)
         queue_name = self._get_param("QueueName")
 
@@ -105,14 +109,14 @@ class SQSResponse(BaseResponse):
         template = self.response_template(GET_QUEUE_URL_RESPONSE)
         return template.render(queue_url=queue.url(request_url))
 
-    def list_queues(self):
+    def list_queues(self) -> str:
         request_url = urlparse(self.uri)
         queue_name_prefix = self._get_param("QueueNamePrefix")
         queues = self.sqs_backend.list_queues(queue_name_prefix)
         template = self.response_template(LIST_QUEUES_RESPONSE)
         return template.render(queues=queues, request_url=request_url)
 
-    def change_message_visibility(self):
+    def change_message_visibility(self) -> Union[str, Tuple[str, Dict[str, int]]]:
         queue_name = self._get_queue_name()
         receipt_handle = self._get_param("ReceiptHandle")
 
@@ -130,66 +134,34 @@ class SQSResponse(BaseResponse):
         template = self.response_template(CHANGE_MESSAGE_VISIBILITY_RESPONSE)
         return template.render()
 
-    def change_message_visibility_batch(self):
+    def change_message_visibility_batch(self) -> str:
         queue_name = self._get_queue_name()
         entries = self._get_list_prefix("ChangeMessageVisibilityBatchRequestEntry")
 
-        success = []
-        error = []
-        for entry in entries:
-            try:
-                visibility_timeout = self._get_validated_visibility_timeout(
-                    entry["visibility_timeout"]
-                )
-            except ValueError:
-                error.append(
-                    {
-                        "Id": entry["id"],
-                        "SenderFault": "true",
-                        "Code": "InvalidParameterValue",
-                        "Message": "Visibility timeout invalid",
-                    }
-                )
-                continue
-
-            try:
-                self.sqs_backend.change_message_visibility(
-                    queue_name=queue_name,
-                    receipt_handle=entry["receipt_handle"],
-                    visibility_timeout=visibility_timeout,
-                )
-                success.append(entry["id"])
-            except ReceiptHandleIsInvalid as e:
-                error.append(
-                    {
-                        "Id": entry["id"],
-                        "SenderFault": "true",
-                        "Code": "ReceiptHandleIsInvalid",
-                        "Message": e.description,
-                    }
-                )
+        success, error = self.sqs_backend.change_message_visibility_batch(
+            queue_name, entries
+        )
 
         template = self.response_template(CHANGE_MESSAGE_VISIBILITY_BATCH_RESPONSE)
         return template.render(success=success, errors=error)
 
-    def get_queue_attributes(self):
+    def get_queue_attributes(self) -> str:
         queue_name = self._get_queue_name()
 
         if self.querystring.get("AttributeNames"):
             raise InvalidAttributeName("")
 
-        attribute_names = self._get_multi_param("AttributeName")
-
         # if connecting to AWS via boto, then 'AttributeName' is just a normal parameter
-        if not attribute_names:
-            attribute_names = self.querystring.get("AttributeName")
+        attribute_names = self._get_multi_param(
+            "AttributeName"
+        ) or self.querystring.get("AttributeName")
 
-        attributes = self.sqs_backend.get_queue_attributes(queue_name, attribute_names)
+        attributes = self.sqs_backend.get_queue_attributes(queue_name, attribute_names)  # type: ignore
 
         template = self.response_template(GET_QUEUE_ATTRIBUTES_RESPONSE)
         return template.render(attributes=attributes)
 
-    def set_queue_attributes(self):
+    def set_queue_attributes(self) -> str:
         # TODO validate self.get_param('QueueUrl')
         attribute = self.attribute
 
@@ -205,7 +177,7 @@ class SQSResponse(BaseResponse):
 
         return SET_QUEUE_ATTRIBUTE_RESPONSE
 
-    def delete_queue(self):
+    def delete_queue(self) -> str:
         # TODO validate self.get_param('QueueUrl')
         queue_name = self._get_queue_name()
 
@@ -214,7 +186,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(DELETE_QUEUE_RESPONSE)
         return template.render()
 
-    def send_message(self):
+    def send_message(self) -> Union[str, Tuple[str, Dict[str, int]]]:
         message = self._get_param("MessageBody")
         delay_seconds = int(self._get_param("DelaySeconds", 0))
         message_group_id = self._get_param("MessageGroupId")
@@ -246,7 +218,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(SEND_MESSAGE_RESPONSE)
         return template.render(message=message, message_attributes=message_attributes)
 
-    def send_message_batch(self):
+    def send_message_batch(self) -> str:
         """
         The querystring comes like this
 
@@ -278,7 +250,7 @@ class SQSResponse(BaseResponse):
 
                 entries[index] = {
                     "Id": value[0],
-                    "MessageBody": self.querystring.get(
+                    "MessageBody": self.querystring.get(  # type: ignore
                         f"SendMessageBatchRequestEntry.{index}.MessageBody"
                     )[0],
                     "DelaySeconds": self.querystring.get(
@@ -317,14 +289,14 @@ class SQSResponse(BaseResponse):
         template = self.response_template(SEND_MESSAGE_BATCH_RESPONSE)
         return template.render(messages=messages, errors=errors)
 
-    def delete_message(self):
+    def delete_message(self) -> str:
         queue_name = self._get_queue_name()
-        receipt_handle = self.querystring.get("ReceiptHandle")[0]
+        receipt_handle = self.querystring.get("ReceiptHandle")[0]  # type: ignore
         self.sqs_backend.delete_message(queue_name, receipt_handle)
         template = self.response_template(DELETE_MESSAGE_RESPONSE)
         return template.render()
 
-    def delete_message_batch(self):
+    def delete_message_batch(self) -> str:
         """
         The querystring comes like this
 
@@ -347,7 +319,7 @@ class SQSResponse(BaseResponse):
                 break
 
             message_user_id_key = f"DeleteMessageBatchRequestEntry.{index}.Id"
-            message_user_id = self.querystring.get(message_user_id_key)[0]
+            message_user_id = self.querystring.get(message_user_id_key)[0]  # type: ignore
             receipts.append(
                 {"receipt_handle": receipt_handle[0], "msg_user_id": message_user_id}
             )
@@ -359,34 +331,18 @@ class SQSResponse(BaseResponse):
                 raise BatchEntryIdsNotDistinct(receipt_and_id["msg_user_id"])
             receipt_seen.add(receipt)
 
-        success = []
-        errors = []
-        for receipt_and_id in receipts:
-            try:
-                self.sqs_backend.delete_message(
-                    queue_name, receipt_and_id["receipt_handle"]
-                )
-                success.append(receipt_and_id["msg_user_id"])
-            except ReceiptHandleIsInvalid:
-                errors.append(
-                    {
-                        "Id": receipt_and_id["msg_user_id"],
-                        "SenderFault": "true",
-                        "Code": "ReceiptHandleIsInvalid",
-                        "Message": f'The input receipt handle "{receipt_and_id["receipt_handle"]}" is not a valid receipt handle.',
-                    }
-                )
+        success, errors = self.sqs_backend.delete_message_batch(queue_name, receipts)
 
         template = self.response_template(DELETE_MESSAGE_BATCH_RESPONSE)
         return template.render(success=success, errors=errors)
 
-    def purge_queue(self):
+    def purge_queue(self) -> str:
         queue_name = self._get_queue_name()
         self.sqs_backend.purge_queue(queue_name)
         template = self.response_template(PURGE_QUEUE_RESPONSE)
         return template.render()
 
-    def receive_message(self):
+    def receive_message(self) -> Union[str, Tuple[str, Dict[str, int]]]:
         queue_name = self._get_queue_name()
         message_attributes = self._get_multi_param("message_attributes")
         if not message_attributes:
@@ -397,7 +353,7 @@ class SQSResponse(BaseResponse):
         queue = self.sqs_backend.get_queue(queue_name)
 
         try:
-            message_count = int(self.querystring.get("MaxNumberOfMessages")[0])
+            message_count = int(self.querystring.get("MaxNumberOfMessages")[0])  # type: ignore
         except TypeError:
             message_count = DEFAULT_RECEIVED_MESSAGES
 
@@ -411,9 +367,9 @@ class SQSResponse(BaseResponse):
             )
 
         try:
-            wait_time = int(self.querystring.get("WaitTimeSeconds")[0])
+            wait_time = int(self.querystring.get("WaitTimeSeconds")[0])  # type: ignore
         except TypeError:
-            wait_time = int(queue.receive_message_wait_time_seconds)
+            wait_time = int(queue.receive_message_wait_time_seconds)  # type: ignore
 
         if wait_time < 0 or wait_time > 20:
             return self._error(
@@ -427,7 +383,7 @@ class SQSResponse(BaseResponse):
         try:
             visibility_timeout = self._get_validated_visibility_timeout()
         except TypeError:
-            visibility_timeout = queue.visibility_timeout
+            visibility_timeout = queue.visibility_timeout  # type: ignore
         except ValueError:
             return ERROR_MAX_VISIBILITY_TIMEOUT_RESPONSE, dict(status=400)
 
@@ -453,7 +409,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(RECEIVE_MESSAGE_RESPONSE)
         return template.render(messages=messages, attributes=attributes)
 
-    def list_dead_letter_source_queues(self):
+    def list_dead_letter_source_queues(self) -> str:
         request_url = urlparse(self.uri)
         queue_name = self._get_queue_name()
 
@@ -462,7 +418,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(LIST_DEAD_LETTER_SOURCE_QUEUES_RESPONSE)
         return template.render(queues=source_queue_urls, request_url=request_url)
 
-    def add_permission(self):
+    def add_permission(self) -> str:
         queue_name = self._get_queue_name()
         actions = self._get_multi_param("ActionName")
         account_ids = self._get_multi_param("AWSAccountId")
@@ -473,7 +429,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(ADD_PERMISSION_RESPONSE)
         return template.render()
 
-    def remove_permission(self):
+    def remove_permission(self) -> str:
         queue_name = self._get_queue_name()
         label = self._get_param("Label")
 
@@ -482,7 +438,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(REMOVE_PERMISSION_RESPONSE)
         return template.render()
 
-    def tag_queue(self):
+    def tag_queue(self) -> str:
         queue_name = self._get_queue_name()
         tags = self._get_map_prefix("Tag", key_end=".Key", value_end=".Value")
 
@@ -491,7 +447,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(TAG_QUEUE_RESPONSE)
         return template.render()
 
-    def untag_queue(self):
+    def untag_queue(self) -> str:
         queue_name = self._get_queue_name()
         tag_keys = self._get_multi_param("TagKey")
 
@@ -500,7 +456,7 @@ class SQSResponse(BaseResponse):
         template = self.response_template(UNTAG_QUEUE_RESPONSE)
         return template.render()
 
-    def list_queue_tags(self):
+    def list_queue_tags(self) -> str:
         queue_name = self._get_queue_name()
 
         queue = self.sqs_backend.list_queue_tags(queue_name)
@@ -811,7 +767,7 @@ ERROR_TOO_LONG_RESPONSE = """<ErrorResponse xmlns="http://queue.amazonaws.com/do
 </ErrorResponse>"""
 
 ERROR_MAX_VISIBILITY_TIMEOUT_RESPONSE = (
-    f"Invalid request, maximum visibility timeout is {MAXIMUM_VISIBILTY_TIMEOUT}"
+    f"Invalid request, maximum visibility timeout is {MAXIMUM_VISIBILITY_TIMEOUT}"
 )
 
 ERROR_INEXISTENT_QUEUE = """<ErrorResponse xmlns="http://queue.amazonaws.com/doc/2012-11-05/">
